"""
Expose as a single library for user to use

"""
from fugle_trade_core.fugle_trade_core import CoreSDK
from fugle_trade.order import OrderObject
from fugle_trade.util import ft_check_password, ft_get_password, ft_set_password


class InitSDK:
    """Main sdk for fugleTrade"""
    def __init__(self, config):
        self.__ip = ""
        self.__AID = config['User']['Account']

    def set_password(self):
        ft_set_password(self.__AID)
        print("Done")



from enum import Enum

class Action(str, Enum):
    Buy = "B" #買
    Sell = "S" #賣


class APCode(str, Enum):
    Common = "1" #整股
    AfterMarket= "2" #盤後
    Odd = "3" #盤後零股
    Emg = "4" #興櫃
    IntradayOdd = "5" #盤中零股

class Trade(str, Enum):
    Cash = "0" #現股
    Margin = "3" #融資
    Short = "4" #融券
    DayTrading = "9" #自動當沖
    DayTradingSell = "A" #現股當沖賣

class PriceFlag(str, Enum):
    Limit = "0" #限價
    Flat = "1" #平盤
    LimitDown = "2" #跌停
    LimitUp = "3" #漲停
    Market = "4" #市價

class BSFlag(str, Enum):
    FOK = "F"
    IOC = "I"
    ROD = "R"



"""
Define Order Objects
"""
import typing
from fugle_trade.constant import (
    Action,
    APCode,
    Trade,
    PriceFlag,
    BSFlag
)

class OrderObject():
    ap_code: APCode
    bs_flag: BSFlag
    price_flag: PriceFlag
    trade: Trade
    buy_sell: Action
    price: float
    stock_no: str
    quantity: int

    def __init__(
            self,
            buy_sell: Action,
            price: float,
            stock_no: str,
            quantity: int,
            ap_code: APCode = APCode.Common,
            bs_flag: BSFlag = BSFlag.ROD,
            price_flag: PriceFlag = PriceFlag.Limit,
            trade: Trade = Trade.Cash,
        ):

        if type(buy_sell) is not Action:
            raise TypeError("Please use fugleTrade.constant Action")

        # fugle_trade_core will check price format, no need to check price here

        if type(stock_no) is not str:
            raise TypeError("Please use type str in stockNo")

        if type(quantity) is not int:
            raise TypeError("Please use type int in quantity")
        elif ap_code == APCode.Common or ap_code == APCode.AfterMarket:
            if quantity < 1 or quantity > 500:
                raise TypeError("quantity must within range 1 ~ 499")

        elif ap_code == APCode.Odd or ap_code == APCode.IntradayOdd:
            if quantity < 1 or quantity > 1000:
                raise TypeError("quantity must within range 1 ~ 999")
        elif ap_code == APCode.Emg:
            if quantity < 1 or quantity > 499000 or (quantity > 1000 and quantity % 1000 != 0):
                raise TypeError("quantity must within range 1 ~ 499000, or multiply of 1000")

        if type(ap_code) is not APCode:
            raise TypeError("Please use fugleTrade.constant APCode")

        if type(bs_flag) is not BSFlag:
            raise TypeError("Please use fugleTrade.constant BSFlag")

        if type(price_flag) is not PriceFlag:
            raise TypeError("Please use fugleTrade.constant PriceFlag")

        if type(trade) is not Trade:
            raise TypeError("Please use fugleTrade.constant Trade")

        self.ap_code = ap_code
        self.price_flag = price_flag
        self.bs_flag = bs_flag
        self.trade = trade
        self.buy_sell = buy_sell
        self.price = price
        self.stock_no = stock_no
        self.quantity = quantity

    def __str__(self):
        return "ap_code: %s, price_flag: %s, bs_flag: %s, trade: %s, buy_sell: %s, price: %s, stock_no: %s, quantity: %s" % (self.ap_code, self.price_flag, self.bs_flag, self.trade, self.buy_sell, self.price, self.stock_no, self.quantity)



"""
handle websocket connection and callback registration
"""

import json
import re
from websocket import WebSocketApp

def convert_to_snakecase(original_dict):
    transformed_dict = {}
    array_items = []
    if not isinstance(original_dict, list):
        for k in original_dict.keys():
            value = re.sub(r'(?<!^)(?=[A-Z])', '_', k).lower()
            if not isinstance(original_dict[k], list):
                if isinstance(original_dict[k], dict):
                    transformed_dict[value] = convert_to_snakecase(original_dict[k])
                else:
                    transformed_dict[value] = original_dict[k]
            else:

                array_items = []
                for i in range(len(original_dict[k])):
                    if isinstance(original_dict[k][i], dict):
                        array_items.append(convert_to_snakecase(original_dict[k][i]))
                        transformed_dict[value] = array_items
                    else:
                        transformed_dict[value] = original_dict[k]
    else:
        array_items = []
        for item in original_dict:
            array_items.append(convert_to_snakecase(item))
        transformed_dict.update(array_items)
    return transformed_dict

class WebsocketHandler():
    """Handle Websocket connection"""
    def __init__(self):
        self.__ws = None
        default_fun = lambda x: print("in default function")
        self.on_order = default_fun
        self.on_dealt = default_fun
        self.on_error = default_fun
        self.on_close = default_fun

    def ws_on_message(self, _, in_message):
        """callback function for websocket message, pipe to order or dealt based on message type"""
        message = json.loads(in_message)
        # if message['data']['$type'] == "System.String" :
        #     return
        try:
            data = json.loads(message['data']['$value'])
            if data['Kind'] == "ACK":
                self.on_order(convert_to_snakecase(data))

            if data['Kind'] == "MAT":
                self.on_dealt(convert_to_snakecase(data))
        except:
            pass

    def ws_on_error(self, _, error):
        """callback function for websocket error"""
        self.on_error(error)

    def ws_on_close(self, _, error):
        """callback function for websocket close"""
        self.on_close(error)

    def set_callback(self, name, func):
        """for upper scope to set different types of callback function"""
        allowed_callback = ['order', 'dealt', 'error', 'close']
        if name in allowed_callback:
            setattr(self, "on_" + name, func)
        else:
            raise Exception("callback " + name + " not allowed")

    def connect_websocket(self, urlEntry):
        """start to connect websocket"""
        self.__ws = WebSocketApp(
            urlEntry,
            on_message=self.ws_on_message,
            on_error=self.ws_on_error,
            on_close=self.ws_on_close
        )
        self.__ws.run_forever()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fugle_trade']

package_data = \
{'': ['*']}

install_requires = \
['fugle-trade-core==0.2.4',
 'keyring==23.5.0',
 'keyrings.cryptfile==1.3.8',
 'websocket-client==1.2.1']

setup_kwargs = {
    'name': 'fugle-trade',
    'version': '0.2.3',
    'description': '',
    'long_description': None,
    'author': 'bistin',
    'author_email': 'bistin@fugle.tw',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
